/**************************************************************/
/*                                                            */
/*   File name :  libspeplot.c                                */
/*                                                            */
/*      Version:   2018-05-17                                 */
/*                                                            */
/*   Copyright (c) 2007-2018 T.Kondo      All Right Reserved  */
/*                                                            */
/**************************************************************/
/*

  this library requires libvssp32

  
2018-05-17
  support old style Y (Power) axis
2018-05-15 
  debug "plotspe_a" in the case of linear power plot mode.
  revise "plotspe" and "plotspe_a" to display bar representing temperature scale in case of linear power mode.
2018-04-09 revise "plotspe_a" to suppres title according to the mode 
2018-04-09 revise "plotspe" and "plotspe_a" to add velocity axis 
2018-02-05 debug of "plotspe" and "plotspe_a" in gnuplot mode 
2017-12-08 "plottime_a" is revised to improve appearance in case of adbit=2
2017-11-13
  divided from libspeplot.h
2015-11-04
  update
2015-10-28
  establish as an independent library
2015-10-27
  "plotspe" and "plotspe_a" is minor updated 
2014-10-29
  "plotspe_a2" is modified to reflect mindbm and maxdbm run parameters.
  "plotspe_a" is modified to reflect frequency range and amplitude range parameters

*/

#include "libspeplot.h"   

void tmpnameget(char *tmpdir, char *fpath)  // original is in libcor.c
// temporary unique name get  2003-07-27
// use mkstemp instead of tmpnam when linux or FreeBSD 2005-09-16
{
    int k;
#ifdef WIN32                                        // 2005-09-16
    char cbuf[L_tmpnam+1];
#else                                               //2005-09-16
    int fd;                                         //2005-09-16
#endif                                              //2005-09-16
    char fname[80],dir[80];
    // check tmdir is accompanied with '/' or '\'
    k=strlen(tmpdir);
    if(k > 0){
       if(tmpdir[k-1] != '/' && tmpdir[k-1] != '\\'){
          tmpdir[k]='/';
          tmpdir[k+1]='\0';
       }
    }
#ifdef WIN32                                        // 2005-09-16
    tmpnam(cbuf);                                
    parse_dir_and_name_fm_fpath(cbuf,dir,fname); 
#else                                               // 2005-09-16
    strcpy(fname,"XXXXXX");                         // 2005-09-16
    fd=mkstemp(fname);                              // 2005-09-16
         // temprary file will be created by "mkstemp"
    close(fd);  // release file descripter          // 2005-09-16
                // to avoid infinit increase of fd  // 2005-09-16
    k=remove(fname);  // delete temporary file      // 2005-09-16
#endif                                              // 2005-09-16
    strcpy(fpath,tmpdir);
    strcat(fpath,fname);
}

/******************************************/
/*    Plot with PGPLOT and GNUPLOT        */
/******************************************/

void plotspe(speana_param_t *st,
             float ymaxdb, float ymindb)
{
    // 2015-10-27  minor update
    // 2018-04-09  add velocity axis mode

    int kplot, m, k;
    double fs, df;    // 2018-05-15
    char fname[120];

    char labfsampl[10];
    char labDeltaF[20]; // 2018-05-15
#ifdef USE_GNUPLOT   // using GNUPLOT
    //char ftemp4ch[4][60];
    char ftemp4ch[st->numch][60];  // 2018-02-05
#endif

    /* replace "\" by "/" if file name includes "\"
       to avoid ugly printing in case of Windows  */
    k=0;
    for(m=0; m< (int)strlen(st->fname); m++){
        fname[k]=st->fname[m];
        if(st->fname[m]=='\\'){
            fname[k]='/';
        }
        k++;
    }
    fname[k]='\0';

    st->xdata = (float *)malloc(st->nfft*sizeof(float));

    /* get frequency resolution for print */
    fs=1000.0*st->sfreq;  // (Hz)
    df=fs/(double)st->nfft;
    if(df < 1000.0){
       sprintf(labDeltaF,"Fres(Hz)= %.1f",df);
    } else if (df < 1.0e6){
       sprintf(labDeltaF,"Fres(kHz)=%.1f",df/1000.0);
    } else {
       sprintf(labDeltaF,"Fres(MHz)=%.1f",df/1.0e6);
    }

    /* sampling frequency in text for print */
    if(st->sfreq < 1000){
        sprintf(labfsampl,"%dkHz",st->sfreq);
    } else {
        sprintf(labfsampl,"%dMHz",st->sfreq/1000);
    }

    for(kplot=0; kplot<2; kplot++){
       if(st->pmode==1 && kplot==1) goto bypass;
       if(st->pmode==2 && kplot==0) goto bypass;
       if(kplot==0) {
           strcpy(st->g_device,st->g_ps_device);
       } else {
           strcpy(st->g_device,st->g_display);
       }

#ifdef USE_GNUPLOT   // using gnuplot
       st->gp= popen(GNUPLOT_EXE,"w");
       if(st->gp==NULL) goto exitend;
       if(kplot==0) {
          //fprintf(gp,"set terminal post eps color\n");
          fprintf(st->gp,"set terminal post landscape 12 color\n");
          fprintf(st->gp,"set output \"%s\"\n",st->ps_fname);
       }
       if(kplot==1) {
          fprintf(st->gp,"set terminal %s\n",st->g_display);
       }
       fflush(st->gp);
       fprintf(st->gp,"set multiplot\n");
       fprintf(st->gp,"set size 0.5,0.5\n");

#else                // using pgplot
       //if(kplot==0) {
       //    strcpy(pgdevice,"%s/PS",st->ps_fname);
       //} else {
       //    strcpy(pgdevice,st->g_device);
       //}
       if(cpgopen(st->g_device) < 1) goto exitend;
       if(strncmp(st->g_device,"/GW",3)==0) cpgeras();   // erase just in case 2007-10-26
       if(strncmp(st->g_device,"/XWIN",5)==0 ||
           strncmp(st->g_device,"/XSER",5)==0 ) { /* set background white 
                                                   and text black */
           cpgscr(0,1.0,1.0,1.0);  /* Background color */
           cpgscr(1,0.0,0.0,0.0);  /* Foreground color */
       }
       if(st->numch != 1){
           cpgsubp(2,2);  // divide plot pannel into 2X2
       }
#endif
       for(m=0; m<st->numch; m++){

#ifdef USE_GNUPLOT   // using gnuplot
           switch(m){
              case 0: fprintf(st->gp,"set origin 0.0,0.5\n"); break;
              case 1: fprintf(st->gp,"set origin 0.5,0.5\n"); break;
              case 2: fprintf(st->gp,"set origin 0.0,0.0\n"); break;
              case 3: fprintf(st->gp,"set origin 0.5,0.0\n"); break;
           }
           //fprintf(gp,"clear\n");   

#endif

           if(st->sekibun < 1.0){
               if(st->sekibun < 1.0e-3){
                  sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(usec)=%6.2f",
                     st->chtable[m],st->adbit,labfsampl,st->sekibun*1.0e6);
               } else {
                  sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(msec)=%.3f",
                     st->chtable[m],st->adbit,labfsampl,st->sekibun*1.0e3);
               }
               strcat(st->lab1,", ");                 // 2018-05-15
               strcat(st->lab1,labDeltaF);            // 2018-05-15
           } else {
                sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(sec)=%.1f",
                   st->chtable[m],st->adbit,labfsampl,st->sekibun);
           }
           strcat(st->lab1,", ");                 // 2018-05-15
           strcat(st->lab1,labDeltaF);            // 2018-05-15


           for(k=0; k<st->nfft; k++) {
               st->xdata[k]=st->spows[k*st->numch+m];
           }

           if(st->amode > 0) {
                st->mode=1;
                plotautocor(st);
           } else {
                plotspe_a(st,ymaxdb,ymindb);
           }
#ifdef USE_GNUPLOT   // using gnuplot
           strcpy(ftemp4ch[m],st->ftemp);
#endif

       }

#ifdef USE_GNUPLOT   // using gnuplot
       fprintf(st->gp,"unset multiplot\n");
       fflush(st->gp);
  #ifdef WIN32
       if(kplot==1) getchar();  // this is necessary to keep display otherwise deleted
  #endif
       pclose(st->gp);
       for(m=0; m<st->numch; m++) remove(ftemp4ch[m]);

#else                // using pgplot
       /* panel undivided to date write */
       cpgsubp(1,1);

       if(st->flag_no_info==0){
           /* date etc write */
           cpgiden();
           /* data file name write (add 2015-11-5)*/
           cpgsch(0.7f);                 // "f" means single precision floating number
           cpgsvp(0.0,1.0,0.0,1.0);   // set viewport as all area
           cpgswin(0.0,1.0,0.0,1.0);   // combine with world corrdinates
           //cpgtext(0.01f,0.01f,st->fname);
           cpgtext(0.01f,0.01f,fname);
       }
       /* close the graphics device */

       if(strncmp(st->g_device,"/GW",3)!=0) cpgclos();   //2007-10-26
#endif
       if(kplot==0){
           printf("Created PostScript file ==> %s\n",st->ps_fname);
       }
 bypass:;
    }
exitend:;
    free(st->xdata);
}

void plotspe2(speana_param_t *st)
{
    /* plot spectrum (or auto cor) POSTSCRIPT OUT and XTERM*/
    int kplot, m, k;
    char labfsampl[10];
    float pmaxdat[4],fdatmax[4];
    int nmax=4;
#ifdef USE_GNUPLOT   // using gnuplot
    //char ftemp4ch[4][60];
    char ftemp4ch[st->numch][60];  // 2018-02-05
#endif


    st->xdata = (float *)malloc(st->nfft*sizeof(float));

    /* sampling frequency in text for print */
    if(st->sfreq < 1000){
        sprintf(labfsampl,"%dkHz",st->sfreq);
    } else {
        sprintf(labfsampl,"%dMHz",st->sfreq/1000);
    }

    for(kplot=0; kplot<2; kplot++){
        if(st->pmode==1 && kplot==1) goto bypass;
        if(st->pmode==2 && kplot==0) goto bypass;
       if(kplot==0) {
           strcpy(st->g_device,st->g_ps_device);
       } else {
           strcpy(st->g_device,st->g_display);
       }
#ifdef USE_GNUPLOT   // using gnuplot
       st->gp= popen(GNUPLOT_EXE,"w");
       if(st->gp==NULL) goto exitend;
       if(kplot==0) {
          //fprintf(gp,"set terminal post eps color\n");
          fprintf(st->gp,"set terminal post landscape 12 color\n");
          fprintf(st->gp,"set output \"%s\"\n",st->ps_fname);
       }
       if(kplot==1) {
          fprintf(st->gp,"set terminal %s\n",st->g_device);
       }
       fflush(st->gp);
       fprintf(st->gp,"set multiplot\n");
       fprintf(st->gp,"set size 0.5,0.5\n");
#else                // using pgplot
        //if(kplot==0) {
        //    strcpy(pgdevice,"/PS");
        //} else {
        //    strcpy(pgdevice,st->g_device);
        //}
        if(cpgopen(st->g_device) < 1) goto exitend;
        if(strncmp(st->g_device,"/GW",3)==0) cpgeras();   // erase just in case 2007-10-26
        if(strncmp(st->g_device,"/XWIN",5)==0 ||
           strncmp(st->g_device,"/XSER",5)==0 ) { /* set background white 
                                                   and text black */
           cpgscr(0,1.0,1.0,1.0);  /* Background color */
           cpgscr(1,0.0,0.0,0.0);  /* Foreground color */
        }

        cpgsubp(2,2);  // divide plot pannel into 2X2
#endif
        for(m=0; m<st->numch; m++){

#ifdef USE_GNUPLOT   // using gnuplot
           switch(m){
              case 0: fprintf(st->gp,"set origin 0.0,0.5\n"); break;
              case 1: fprintf(st->gp,"set origin 0.5,0.5\n"); break;
              case 2: fprintf(st->gp,"set origin 0.0,0.0\n"); break;
              case 3: fprintf(st->gp,"set origin 0.5,0.0\n"); break;
           }

#endif
           sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(sec)=%.1f",
               //m+1,st->adbit,labfsampl,st->sekibun);
               st->chtable[m],st->adbit,labfsampl,st->sekibun);
                
           for(k=0; k<st->nfft; k++) st->xdata[k]=st->spows[k*st->numch+m];

           if(st->amode > 0) {
              st->mode=1;
              plotautocor(st);
           } else {
              plotspe_a2(st,  pmaxdat, fdatmax, nmax);
              if(kplot==0){
                 printf("1st Max %fdbm at %fkHz\n",pmaxdat[0],fdatmax[0]);
                 printf("2nd Max %fdbm at %fkHz\n",pmaxdat[1],fdatmax[1]);
                 printf("3rd Max %fdbm at %fkHz\n",pmaxdat[2],fdatmax[2]);
                 printf("4th Max %fdbm at %fkHz\n",pmaxdat[3],fdatmax[3]);
              }
           }
#ifdef USE_GNUPLOT   // using gnuplot
           strcpy(ftemp4ch[m],st->ftemp);
#endif
        }

#ifdef USE_GNUPLOT   // using gnuplot
       fprintf(st->gp,"unset multiplot\n");
       fflush(st->gp);
  #ifdef WIN32
       if(kplot==1) getchar();  // this is necessary to keep display otherwise deleted
  #endif
       pclose(st->gp);
       for(m=0; m<st->numch; m++) remove(ftemp4ch[m]);
#else                // using pgplot
        /* panel undivided to date write */
       cpgsubp(1,1);
       /* date etc write */
       cpgiden();
       /* close the graphics device */
       if(strncmp(st->g_device,"/GW",3)!=0) cpgclos();   //2007-10-26
#endif
       if(kplot==0){
           printf("Created PostScript file ==> %s\n",st->ps_fname);
       }
    bypass:;
    }

exitend:;
    free(st->xdata);
}


void plotspe_a(speana_param_t *st, float ymaxdb, float ymindb)
/*
               mode2 -- YX: axis scale mode
                          X: frequency axis mode
                             =0: log scale  =1: linear scale
                          Y: amplitude axis mode
                             =0: log scale 
                             =1: linear scale (common for 4 channels)
                                  (use ymaxdb, ymindb)
                             =2: linear scale (each max and min)
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin;
      float vmax, vmin, rf0, rf1, deltaf;    // 2018-04-09
      float fscale;
      float *fdat, *powdat;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax;
      int mode,ymode;
      float xpos,ypos,tbar_w, tbar_x,tbar_y; // 2018-05-15
      float xdat[2],ydat[2]; // 2018-05-15
      char clab[10];  // 2018-05-15

      ymode=st->mode2/10;
      mode=st->mode2 % 10;
      strcpy(htitl,"POWER SPECTRUM");



      /* frequency axis generation */

      //npmax=npos/2;
      npmax=st->nfft/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      
      fsampl=(float)1000.0*(float)st->sfreq;  // (Hz)
      df=fsampl/(float)st->nfft;
      if(st->flag_velocity==0){
          /* frequency axis */
          if(st->sfreq > 8000){  // if sampling frequency exceeds 8 MHz
              fscale = 1.0e6;  // scaling in MHz
              strcpy(xtit,"FREQUENCY (MHz)");
          } else {
              fscale = 1000.0; // scaling in kHz
              strcpy(xtit,"FREQUENCY (kHz)");
          }
          if(mode == 0) {    // log scale
#ifdef USE_GNUPLOT   // using gnuplot
             fmax=(float)(fsampl/(2.0*fscale));
             fmin=(float)(df/fscale);
             for(n=0; n<npmax; n++)fdat[n]=(float)((float)n*df/fscale); 
#else // use PGPLOT
             fmax=(float)log10(fsampl/(2.0*fscale));
             fmin=(float)log10(df/fscale);
             for(n=0; n<npmax; n++) fdat[n]=(float)log10((float)n*df/fscale);
#endif
          } else {          //  linear scale
             fmax=(float)(fsampl/(2.0*fscale));
             fmin=0.0;
             if(st->f1khz != 0.0) fmin=(float)(st->f1khz/(fscale/1000.0));
             if(st->f2khz != 0.0) fmax=(float)(st->f2khz/(fscale/1000.0));
             for(n=0; n<npmax; n++){fdat[n]=(float)((float)n*df/fscale); }
          }

      } else {    
          /* velocity axis */  // added on 2018-04-09
          if(st->rfmhz >= 0.0){   // USB
             rf0=st->rfmhz*1.0e6;
             rf1=rf0+0.5*fsampl;
          } else {            // LSB
             rf0=fabs(st->rfmhz)*1.0e6;
             rf1=rf0-0.5*fsampl;
          }

          fscale = 1000.0;  // scaling for km/s
          strcpy(xtit,"VELOCITY (km/s)");

          if(st->rfmhz >= 0.0){
             fmin=rf0;
             fmax=rf1;
             if(st->f1khz != 0.0) fmin=rf0+(float)(st->f1khz*1000.0);
             if(st->f2khz != 0.0) fmax=rf0+(float)(st->f2khz*1000.0);
          } else {
             fmin=rf0;
             fmax=rf1;
             if(st->f1khz != 0.0) fmin=rf0-(float)(st->f1khz*1000.0);
             if(st->f2khz != 0.0) fmax=rf0-(float)(st->f2khz*1000.0);
             //fmin=rf1;
             //fmax=rf0;
             //if(st->f1khz != 0.0) fmin=rf0-(float)(st->f1khz*1000.0);
             //if(st->f2khz != 0.0) fmax=rf0-(float)(st->f2khz*1000.0);
          }
          if(st->refmhz != 0.0){ 
              vmin=(float)((VLIGHT*(fmin-st->refmhz*1.0e6)/(st->refmhz*1.0e6))/fscale) ; 
              vmax=(float)((VLIGHT*(fmax-st->refmhz*1.0e6)/(st->refmhz*1.0e6))/fscale) ; 
          } else {
              vmin=(float)((VLIGHT*(fmin-st->refmhz*1.0e6)/(st->rfmhz*1.0e6))/fscale) ; 
              vmax=(float)((VLIGHT*(fmax-st->refmhz*1.0e6)/(st->rfmhz*1.0e6))/fscale) ; 
          }

          fmin=vmin;
          fmax=vmax;

          // linear scale only
          for(n=0; n<npmax; n++){
              if(st->rfmhz >= 0.0){   // USB
                  deltaf=rf0+(float)n*df-st->refmhz*1.0e6;
              } else {
                  deltaf=rf0-(float)n*df-st->refmhz*1.0e6;
              }
              if(st->refmhz != 0.0){ 
                  fdat[n]=(float)((VLIGHT*deltaf/(st->refmhz*1.0e6))/fscale) ; 
              } else {             
                  fdat[n]=(float)((VLIGHT*deltaf/(st->rfmhz*1.0e6))/fscale) ; 
              }
          }
      }


      /* amplitude axis */

      /* data copy, if necessary convert to linear scale */
      if(ymode == 0){    // log scale
         for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];
      } else {   // linear scale
         /* convert to linear scale */
         for(n=0; n<npmax; n++) powdat[n]=(float)pow(10.0,st->xdata[n]/10.0);
      }

      /* data max, min check */
      for(n=0; n<npmax; n++){
         //powdat[n]=spow[n];
         if(n==0){
            pmax=powdat[n];
            pmin=powdat[n];
         } else {
            if(powdat[n] > pmax) pmax=powdat[n];
            if(powdat[n] < pmin) pmin=powdat[n];
         }
      }

      if(ymode == 0){    // log scale
         strcpy(ytit,"POWER (dBm)");
         if( st->adbit==1 ) strcpy(ytit,"POWER (dB)"); 
         ymax=10.0;
         ymin=-80.0;
         //if (st->adbit >= 8 ) ymin=-100;     // extend dynamic range (2015-10-27 to support AD 16 bits)
         if(st->maxdbm!=0.0){     // 2014-10-29
             ymax=(float)st->maxdbm;
         }
         if(st->mindbm!=0.0){     // 2014-10-29
             ymin=(float)st->mindbm;
         }

      } else {   // linear scale
         strcpy(ytit,"POWER (mW)");                   // 2018-05-15
         if( st->adbit==1 ) strcpy(ytit,"POWER");     // 2018-05-15
         ymax=pmax;
         ymin=pmin;
         if(ymode==1){
            ymax=(float)pow(10.0,ymaxdb/10.0);
            ymin=(float)pow(10.0,ymindb/10.0);
            if(st->maxlin!=99.0){     // 2015-05-15
               ymax=(float)st->maxlin;
            }
            if(st->minlin!=99.0){     // 2015-05-15
               ymin=(float)st->minlin;
            }
         }
      }

#ifdef USE_GNUPLOT   // using gnuplot

      /* define coordinate range of graph */
      if(mode == 0){   
           fprintf(st->gp,"set logscale x\n");
      } else {         
           fprintf(st->gp,"unset logscale\n");
      }

      fprintf(st->gp,"set xrange [%g:%g]\n",fmin,fmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO TIME VARIABLE\" at graph 0.5, 0.52 center\n");
          fprintf(st->gp,"set label \"COMPONENT FOUND\" at graph 0.5, 0.48 center\n");
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  
          gnline2(st->gp,2,fdat,powdat,st->ftemp); 
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          gnline2(st->gp,npmax,fdat,powdat,st->ftemp); 

      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else  // using PGPLOT

      //cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgsch(1.5);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width


//      if(ymode==1){         // 2018-05-15    linear power scale
//          cpgsch(1.5);       
//      }
      /* define coordinate range of graph */

      if(st->ystyle_old==1){
         if(mode == 0){   
            cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // X axis log scale
         } else {     
            cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // X axis linear scale
         }
      } else {

         cpgpage();  // this is necessary to change panel in case of not use of "cpgenv" function
         //cpgsvp(0.17,0.9,0.15,0.85);
         cpgsvp(0.2,0.92,0.15,0.85);
         //cpgenv(fmin, fmax, ymin, ymax, 0, -1);  
         cpgswin(fmin, fmax, ymin, ymax);  
         if(mode == 0){   
             cpgbox("BCLNST",0.0,0,"BCNSTV",0.0,0);  // X axis log scale
         } else {     
             cpgbox("BCNST",0.0,0,"BCNSTV",0.0,0);  // X axis log scale
         }
      }
      

//      if(ymode==1){         // 2018-05-15    linear power scale
//          cpgsch(1.5);       
//      } else {
//          cpgsch(2.0);  // set text height 2.0  (default is 1)
//      }

      /* top title and label the axes */

      if(st->ystyle_old==1){
         cpglab("",ytit,"");           // Y-axis
      } else {
         //cpglab("",ytit,"");           // Y-axis
         if(ymode==1){         // 2018-05-15    linear power scale
            //cpgmtxt("L",5.0,0.5,0.5,ytit);   // Y axis label
            cpgmtxt("L",5.5,0.5,0.5,ytit);   // Y axis label
         } else {
            cpgmtxt("L",4.0,0.5,0.5,ytit);   // Y axis label 
         }
      }

      cpgmtxt("B",2.5,0.5,0.5,xtit);   // X axis 

      /* top title write*/
      cpgsch(2.0);  // set text height 2.0  (default is 1)
      if(st->flag_no_title==0){
          cpglab("","",htitl);           // top title
      }
      /* sub label write*/
      cpgsch(1.5);
      if(st->flag_no_title==0 && st->mode_title!=2){   // 2018-04-10
         cpgmtxt("T",1.5,0.5,0.5,st->lab1);
      }
      if(st->flag_no_title==0 && st->mode_title==0){   // 2018-04-10
          cpgmtxt("T",0.5,0.5,0.5,st->lab2);
      }
      cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }

      /* plot temperature bar at (xpos, ypos)  2018-05-16*/
      //xpos=0.8;
      //ypos=0.75;
      xpos=st->tbar_xpos;    // ranging 0.0 to 1.0
      ypos=st->tbar_ypos;    // ranging 0.0 to 1.0
      tbar_w=(fmax-fmin)/40.0;
      //if(st->tbar_p2_K >= 1.0 && fmod(st->tbar_p2_K,1.0) < 0.01){
      //    sprintf(clab,"%dK",(int)st->tbar_p2_K);
      //} else {
      //    sprintf(clab,"%fK",st->tbar_p2_K);
      //}
      sprintf(clab,"%sK",st->tbar_p2_K);
      if(ymode == 1 && st->tbar_p1_mW > 0.0){   // linear scale
          cpgsch(1.5);
          tbar_x=fmin+xpos*(fmax-fmin);    // X position of T bar
          tbar_y=ymin+ypos*(ymax-ymin)-0.5*st->tbar_p1_mW;   // Y position of bottom of T bar
          xdat[0]=tbar_x; xdat[1]=tbar_x;
          ydat[0]=tbar_y; ydat[1]=ydat[0]+st->tbar_p1_mW;
          cpgline(2,xdat,ydat); 
          xdat[0]=tbar_x-0.5*tbar_w; xdat[1]=tbar_x+0.5*tbar_w;
          ydat[0]=tbar_y; ydat[1]=ydat[0];
          cpgline(2,xdat,ydat); 
          ydat[0]=tbar_y+st->tbar_p1_mW; ydat[1]=ydat[0];
          cpgline(2,xdat,ydat);
          //
          cpgptxt(tbar_x+0.5*tbar_w,tbar_y+0.45*st->tbar_p1_mW,0.0,0.0,clab);
      }
   
#endif

     free(powdat);
     free(fdat);
}

void plotspe_a2(speana_param_t *st,float *pmaxdat, 
               float *fdatmax, int nmax)
/*
        st->mode = 0; log frequency scale   1; linear frequency scale
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float fscale;
      float *fdat, *powdat;
      float fatmax;
      char htitl[60], xtit[60], ytit[60];
      int n, npmax,k, ncount,nmaxpos;
      //float pmaxdat[4],fdatmax[4];
      //nmax=4;
      strcpy(htitl,"POWER SPECTRUM");

      /* frequency axis generation */

      npmax=st->nfft/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));

      fsampl=(float)(1000.0*(float)st->sfreq);
      df=fsampl/(float)st->nfft;
      if(st->sfreq > 8000){  // if sampling frequency exceeds 8 MHz
          fscale = 1.0e6;  // scaling in MHz
          strcpy(xtit,"FREQUENCY (MHz)");
      } else {
          fscale = 1000.0; // scaling in kHz
          strcpy(xtit,"FREQUENCY (kHz)");
      }
      if(st->mode == 0) {    // log scale
         fmax=(float)log10(fsampl/(2.0*fscale));
         fmin=(float)log10(df/fscale);
         for(n=0; n<npmax; n++){fdat[n]=(float)log10((float)n*df/fscale); }
      } else {          //  linear scale
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=0.0;
         //fmaxkhz=990.0;
         //fminkhz=920.0;
         if(st->f1khz != 0.0) fmin=(float)(st->f1khz/(fscale/1000.0));
         if(st->f2khz != 0.0) fmax=(float)(st->f2khz/(fscale/1000.0));
         for(n=0; n<npmax; n++) fdat[n]=(float)(n*df/fscale); 
      }

      strcpy(ytit,"POWER (dBm)");
      if( st->adbit==1 ) strcpy(ytit,"POWER (dB)"); 
      ymax=10.0;
      ymin=-80.0;
      //ymax=-34.0;
      //ymin=-40.0;
      if (st->adbit == 8 ){ ymin=-100; }    // extend dynamic range 

      /* data max, min check */
      //printf("max search start %d\n",npmax);
      
      for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];
    
      pmin= (float)1.0e30;
      for(ncount=0; ncount<nmax; ncount++){
          pmax=(float)(-1.0e30);
          k=0;
          for(n=0; n<npmax; n++){
            if(fdat[n]>=fmin && fdat[n]<=fmax){
               if(powdat[n] > pmax){ 
                   pmax=powdat[n];
                   fatmax=fdat[n];
                   pmaxdat[ncount]=pmax;
                   fdatmax[ncount]=(float)(fatmax*(fscale/1000.0));  // to obtain in kHz
                   nmaxpos=n;
               }
               if(ncount==0) {
                    if(powdat[n] < pmin) pmin=powdat[n]; 
               }
               k++;
            }
          }
         powdat[nmaxpos]=(float)(-10.0e30);
         //printf("pmax,fatmax, i %f %f %d\n",pmax,fatmax,i);
      }
     /*
      printf("Number of points = %d\n",k);
      printf("1st Max %fdbm at %fkHz\n",pmaxdat[0],fdatmax[0]);
      printf("2nd Max %fdbm at %fkHz\n",pmaxdat[1],fdatmax[1]);
      printf("3rd Max %fdbm at %fkHz\n",pmaxdat[2],fdatmax[2]);
      printf("4th Max %fdbm at %fkHz\n",pmaxdat[3],fdatmax[3]);
     */
      maxdbm=pmaxdat[0];
      mindbm=pmin;
      if(st->maxdbm==0.0){     // 2014-10-29
          ymax=pmaxdat[0];
      } else { 
          ymax=(float)st->maxdbm;
      }
      if(st->mindbm==0.0){     // 2014-10-29
          ymin=pmin;
      } else {
          ymin=(float)st->mindbm;
      }
      //printf("ymax,ymin %f %f\n",ymax,ymin);
    
      for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];

#ifdef USE_GNUPLOT   // using gnuplot

      /* define coordinate range of graph */
      if(st->mode == 0){   
         //cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
          fprintf(st->gp,"set logscale x\n");
      } else {         
         //cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
          fprintf(st->gp,"unset logscale\n");
      }
      fprintf(st->gp,"set xrange [%g:%g]\n",fmin,fmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      //cpglab(xtit,ytit,htitl);
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO TIME VARIABLE\" at graph 0.5, 0.52 center\n");
          fprintf(st->gp,"set label \"COMPONENT FOUND\" at graph 0.5, 0.48 center\n");
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  
          gnline2(st->gp,2,fdat,powdat,st->ftemp); 
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          gnline2(st->gp,npmax,fdat,powdat,st->ftemp); 

      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else   // use PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(st->mode == 0){   
         cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
       cpglab(xtit,ytit,htitl);

      /* sub label write*/
       cpgsch(1.5);
       cpgmtxt("T",1.5,0.5,0.5,st->lab1);
       cpgmtxt("T",0.5,0.5,0.5,st->lab2);
       cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }

#endif

     free(powdat);
     free(fdat);
}


void plotautocor(speana_param_t *st)
/*
        st->mode = 0; log frequency scale   1; linear frequency scale
*/
{
      //float cofst,tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin, maxdbm, mindbm;
      float tmax, tmin, fsampl, dt, ymax, ymin, pmax, pmin;
      float *tdat, *pdat;
      //float x,y;
      char htitl[60], xtit[60], ytit[60];
      int n, k, npmax, nhalf;

      strcpy(htitl,"AUTO CORRELATION (ABS)");

      /* time axis generation */

      if(st->amode <= 1) {
         npmax=st->nfft;
      } else {
         npmax=st->amode;
      }
      tdat = (float *)malloc(npmax*sizeof(float));
      pdat = (float *)malloc(npmax*sizeof(float));
      nhalf=npmax/2;

      fsampl=(float)(1000.0*(float)st->sfreq);
      dt=(float)(1.0/fsampl);
      for(n=0; n<npmax; n++){
          tdat[n]=(float)((float)(n-nhalf)*dt*1.0e6);   // usec unit
      }
      tmax=tdat[npmax-1];
      tmin=tdat[0];
      //printf("Debug %d %f %f\n",st->sfreq,tmax,tmin);

      strcpy(xtit,"DELAY TIME (us)");
      strcpy(ytit,"CORRELATION AMP");
      ymax=(float)1.1;
      ymin=0.0;

      /* data range limited copy and max, min check */
     
      for(n=0; n<npmax; n++){
         k=n-nhalf;
         if(k < 0) k=st->nfft+k;
         pdat[n]=st->xdata[k];
         if(pdat[n]<0) pdat[n]=-pdat[n];  // to take absolute value
         if(n==0){
            pmax=pdat[n];
            pmin=pdat[n];
         } else {
            if(pdat[n] > pmax) pmax=pdat[n];
            if(pdat[n] < pmin) pmin=pdat[n];
         }
      }

#ifdef USE_GNUPLOT   // using gnuplot
      

      /* define coordinate range of graph */
      if(st->mode == 0){   
           fprintf(st->gp,"set logscale x\n");
      } else {         
           fprintf(st->gp,"unset logscale\n");
      }
      fprintf(st->gp,"set xrange [%g:%g]\n",tmin,tmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.13 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO DATA FOR PLOT\" at graph 0.5, 0.5 center\n");
          tdat[0]=tdat[1];
          pdat[0]=pdat[1];  
          gnline2(st->gp,2,tdat,pdat,st->ftemp); // dummy plot to get coordinate box
          
      } else {
      
        /* plot the line graph */
         gnline2(st->gp,npmax,tdat,pdat,st->ftemp); 
      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else  // using PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      /* define coordinate range of graph */
      if(st->mode == 0){   
         cpgenv(tmin, tmax, ymin, ymax, 0, 10);   // log scale
      } else {         
         cpgenv(tmin, tmax, ymin, ymax, 0, 0);   // linear scale
      }
      
      /* label the axes */
      cpglab(xtit,ytit,htitl);

      /* sub label write*/
      cpgsch(1.5);
      cpgmtxt("T",1.5,0.5,0.5,st->lab1);
      cpgmtxt("T",0.5,0.5,0.5,st->lab2);
      cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO DATA FOR PLOT");
          
      } else {

          cpgline(npmax,tdat,pdat); 

      }

#endif

     free(pdat);
     free(tdat);
}


/******************************************/
/*    Plot with GNUPLOT                   */
/******************************************/

void gnline(FILE *gp,  int ndata,float *xdata, float *ydata)
{
    /* donot use this function for multiplot
      use gnline2 for multi plot
    */
    int n;
    char *ftemp="gnplot.tmp"; 
    FILE *fo;

    if( (fo  = fopen( ftemp,"w" )) == NULL ){
        printf(" File (%s) Open Error! \n",ftemp);
        goto exit1;
    } else {
        //printf(" File (%s) Opened for GNUPLOT data write! \n",ftemp);
        for(n=0; n<ndata; n++){
            fprintf(fo, "%g %g\n",xdata[n],ydata[n]);
        }
    }
    //fflush(fo);
    fclose(fo);
    //tk_sleep(10);  // suspend 1 msec 
    fprintf(gp,"plot \"%s\" with lines notitle lt -1 lw 0.5\n",ftemp);
    fflush(gp);
exit1:;
}

void gnline2(FILE *gp,int ndata,float *xdata, float *ydata, char *ftemp)
{
 /*
   [in]
      gp  --- pointer to popened stream
      ndata --  number of points in xdata and ydata
      xdata[ndata] --  X data
      ydata[ndata] --  Y data
   [return]
      ftemp        --  file name used for line data to interface to GNUPLOT
                       should be deleted after pclose(gp)
*/

    int n;
    FILE *fo;
#ifdef WIN32
    char ftemptab[][20]={"gnuplot1.tmp","gnuplot2.tmp","gnuplot3.tmp","gnuplot4.tmp"};
    strcpy(ftemp,ftemptab[tempcounter]);
    tempcounter++;        // tempcounter is a gloval variable
    if(tempcounter >= 4) tempcounter=0;
#else
    tmpnameget("./", ftemp);  // original is in libcor.c
#endif
    if( (fo  = fopen( ftemp,"w" )) == NULL ){
        printf(" File (%s) Open Error! \n",ftemp);
        goto exit1;
    } else {
        //printf(" File (%s) Opened for GNUPLOT data write! \n",ftemp);
        for(n=0; n<ndata; n++){
            fprintf(fo, "%g %g\n",xdata[n],ydata[n]);
        }
    }
    fclose(fo);
    fprintf(gp,"plot \"%s\" with lines notitle lt -1 lw 0.5\n",ftemp);
    fflush(gp);
    //remove(ftemp);  //should be deleted after pclose(gp)
exit1:;

}


/*  added 2015-10-28 */


void plottime(speana_param_t *st)
{
/* 
       plot time series data
             2015-10-28
*/
    int kplot, m, k, n;
    char labfsampl[10];
#ifdef USE_GNUPLOT   // using GNUPLOT
    char ftemp4ch[4][60];
#endif

    //st->xdata = (float *)malloc(st->nfft*sizeof(float));
    //st->tdat = (float *)malloc(st->nfft*sizeof(float));
    st->xdata = (float *)malloc(st->nadata*sizeof(float));
    st->tdat = (float *)malloc(st->nadata*sizeof(float));



    /* prepare time axis data */
    for(n=0; n< st->nadata; n++){
        st->tdat[n]=(float)n/(float)(st->sfreq*1000.0);
    }

    /* sampling frequency in text for print */
    if(st->sfreq < 1000){
        sprintf(labfsampl,"%dkHz",st->sfreq);
    } else {
        sprintf(labfsampl,"%dMHz",st->sfreq/1000);
    }

    for(kplot=0; kplot<2; kplot++){
       if(st->pmode==1 && kplot==1) goto bypass;
       if(st->pmode==2 && kplot==0) goto bypass;
       if(kplot==0) {
           strcpy(st->g_device,st->g_ps_device);
       } else {
           strcpy(st->g_device,st->g_display);
       }

#ifdef USE_GNUPLOT   // using gnuplot
       st->gp= popen(GNUPLOT_EXE,"w");
       if(st->gp==NULL) goto exitend;
       if(kplot==0) {
          //fprintf(gp,"set terminal post eps color\n");
          fprintf(st->gp,"set terminal post landscape 12 color\n");
          fprintf(st->gp,"set output \"%s\"\n",st->ps_fname);
       }
       if(kplot==1) {
          fprintf(st->gp,"set terminal %s\n",st->g_display);
       }
       fflush(st->gp);
       fprintf(st->gp,"set multiplot\n");
       fprintf(st->gp,"set size 0.5,0.5\n");

#else                // using pgplot
       //if(kplot==0) {
       //    strcpy(pgdevice,"%s/PS",st->ps_fname);
       //} else {
       //    strcpy(pgdevice,st->g_device);
       //}
       if(st->flag_g_device_open==0){
           if(cpgopen(st->g_device) < 1) goto exitend;
           st->flag_g_device_open=1;
       }
       if(strncmp(st->g_device,"/GW",3)==0) cpgeras();   // erase just in case 2007-10-26
       if(strncmp(st->g_device,"/XWIN",5)==0 ||
           strncmp(st->g_device,"/XSER",5)==0 ) { /* set background white 
                                                   and text black */
           cpgscr(0,1.0,1.0,1.0);  /* Background color */
           cpgscr(1,0.0,0.0,0.0);  /* Foreground color */
       }

       if(st->numch != 1){
           cpgsubp(2,2);  // divide plot pannel into 2X2
       }
#endif
       for(m=0; m<st->numch; m++){

#ifdef USE_GNUPLOT   // using gnuplot
           switch(m){
              case 0: fprintf(st->gp,"set origin 0.0,0.5\n"); break;
              case 1: fprintf(st->gp,"set origin 0.5,0.5\n"); break;
              case 2: fprintf(st->gp,"set origin 0.0,0.0\n"); break;
              case 3: fprintf(st->gp,"set origin 0.5,0.0\n"); break;
           }
           //fprintf(gp,"clear\n");   

#endif

           if(st->tspan < 1.0){ 
               if(st->tspan < 1.0e-3){
                  sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, T_span(usec)=%6.2f",
                    // m+1,st->adbit,labfsampl,st->tspan*1.0e6);
                    st->chtable[m],st->adbit,labfsampl,st->tspan*1.0e6);
               } else {
                  sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, T_span(msec)=%.3f",
                     //m+1,st->adbit,labfsampl,st->tspan*1.0e3);
                     st->chtable[m],st->adbit,labfsampl,st->tspan*1.0e3);
               }
           } else {
                sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, T_span(sec)=%.1f",
                   //m+1,st->adbit,labfsampl,st->tspan);
                   st->chtable[m],st->adbit,labfsampl,st->tspan);
           }
           for(k=0; k<st->nadata; k++) {
                st->xdata[k]=st->adata[k*4+m];
           }

           plottime_a(st,m);
#ifdef USE_GNUPLOT   // using gnuplot
           strcpy(ftemp4ch[m],st->ftemp);
#endif

       }

#ifdef USE_GNUPLOT   // using gnuplot
       fprintf(st->gp,"unset multiplot\n");
       fflush(st->gp);
  #ifdef WIN32
       if(kplot==1) getchar();  // this is necessary to keep display otherwise deleted
  #endif
       pclose(st->gp);
       for(m=0; m<st->numch; m++) remove(ftemp4ch[m]);

#else                // using pgplot
       /* panel undivided to date write */
       cpgsubp(1,1);
       /* date etc write */
       cpgiden();
       /* data file name write */
       cpgsch(0.7f);                 // "f" means single precision floating number
       cpgsvp(0.0,1.0,0.0,1.0);   // set viewport as all area
       cpgswin(0.0,1.0,0.0,1.0);   // combine with world corrdinates
       cpgtext(0.01f,0.01f,st->fname);
       /* close the graphics device */
       if(strncmp(st->g_device,"/GW",3)!=0){
           cpgclos();   //2007-10-26
           st->flag_g_device_open=0;
       }
#endif
       if(kplot==0){
           printf("Created PostScript file ==> %s\n",st->ps_fname);
       }
 bypass:;
    }
exitend:;
    free(st->xdata);
    free(st->tdat);
}



void plottime_a(speana_param_t *st, int mch)
/*

           2017-12-08  revised to improve appearance in case of adbit=2

               mode2 -- YX: axis scale mode
                          X: frequency axis mode
                             =0: log scale  =1: linear scale
                          Y: amplitude axis mode
                             =0: log scale 
                             =1: linear scale (common for 4 channels)
                                  (use ymaxdb, ymindb)
                             =2: linear scale (each max and min)
*/
{
      float tmax, tmin, fsampl, amax, amin;
      float tmax0,tmin0, tmaxr,tminr;
      float *tdat, *adat;
      float thaba,tscale;
      char htitl[60], xtit[60], ytit[60];
      char lab2[128];
      char cw;
      int n, npmax;

      htitl[0]='\0';
      //strcpy(htitl,"TIME SERIES");

      //npmax=st->usampl;
      npmax=st->nadata;

      tdat   = (float *)malloc(npmax*sizeof(float));
      adat = (float *)malloc(npmax*sizeof(float));

      /* time axis */
      fsampl=(float)1000.0*(float)st->sfreq;
      thaba=(float)npmax/fsampl;
      tscale=1.0;

      strcpy(xtit,"TIME (sec)");
      sprintf(lab2,"%s offset(msec)=%10.3f",st->lab2,st->toffset*1.0e3);

      for(n=0; n<npmax; n++) tdat[n]=(float)n/fsampl;
      tmax0=tdat[npmax-1];
      tmin0=tdat[0];

      
      for(n=0; n<npmax; n++) tdat[n]*=tscale;

      tmin=tmin0;
      tmax=tmax0;

      if(tmax > st->tspan) tmax=(float)st->tspan;
      //tmax*=tscale;
      //tmin*=tscale;


      /* amplitude axis */

      /* data copy */
      for(n=0; n<npmax; n++) adat[n]=st->xdata[n];

      /* data max, min check */
//      for(n=0; n<npmax; n++){
//         if(n==0){
//            amax=adat[n];
//            amin=adat[n];
//         } else {
//            if(adat[n] > amax) amax=adat[n];
//            if(adat[n] < amin) amin=adat[n];
//         }
//      }

      amin=0.0;
      amax=(float)pow(2,(double)st->adbit);
      if(st->adbit==1){
          amin=(float)(-0.2);
          amax=(float)1.2;
      } else if(st->adbit==2){        // added on 2017-12-08
          amin=(float)(-0.2);         // added on 2017-12-08
          amax=(float)3.2;            // added on 2017-12-08
      }

      strcpy(ytit,"AMPLITUDE");

loop:
      tmaxr=tmax*tscale;
      tminr=tmin*tscale;


#ifdef USE_GNUPLOT   // using gnuplot

      /* define coordinate range of graph */
      fprintf(st->gp,"unset logscale\n");

      fprintf(st->gp,"set xrange [%g:%g]\n",tminr,tmaxr);
      fprintf(st->gp,"set yrange [%g:%g]\n",amin,amax);
      
      /* label the axes */
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",lab2);
      }

      /* check data */
      if (amax == amin){
          fprintf(st->gp,"set label \"NO TIME VARIABLE\" at graph 0.5, 0.52 center\n");
          fprintf(st->gp,"set label \"COMPONENT FOUND\" at graph 0.5, 0.48 center\n");
          tdat[0]=tdat[1];
          adat[0]=adat[1];  
          gnline2(st->gp,2,tdat,adat,st->ftemp); 
          
      } else {

          /* plot the line graph */
          tdat[0]=tdat[1];
          adat[0]=adat[1];  // to avoid ugry plot
          gnline2(st->gp,npmax,tdat,adat,st->ftemp); 

      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else  // using PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      cpgpanl((mch % 2)+1,(mch/2+1));  // when use this, should not use cpgenv
                                           //    mch    ix      iy        cpgpanl(ix,iy)
                                           //     0      1       1
                                           //     1      2       1
                                           //     2      1       2
                                           //     3      2       2

      cpgeras();  // doesn't work properly with cpgenv

      /* define coordinate range of graph */
//      cpgenv(tminr, tmaxr, amin, amax, 0, 0);   // <= this doesn't work well when cpgpanl is called
                                                  //

      //cpgsvp(0.4,0.9,0.2,0.2);  // this doesn't work for sub panel
      cpgvstd();
      cpgswin(tminr, tmaxr, amin, amax);          //  these two lines work as same as cpgenv()
      //cpgbox("BCNTS",0.0,0,"BCNTSV",0.0,0);        //
      cpgbox("BCNTS",0.0,0,"BCNTS",0.0,0);        //


      /* label the axes */
      cpglab(xtit,ytit,htitl);

      /* sub label write*/
      cpgsch(1.5);
      cpgmtxt("T",1.5,0.5,0.5,st->lab1);
      //cpgmtxt("T",0.5,0.5,0.5,st->lab2);
      cpgmtxt("T",0.5,0.5,0.5,lab2);
      cpgsch(2.0);

      /* check data */
      if (amax == amin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          tdat[0]=tdat[1];
          adat[0]=adat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,tdat,adat); 

      }
   
#endif

      if(tmax < tmax0){
         tmin=tmax;
         tmax+=(float)st->tspan;
         if(st->flag_halt==1){
             printf(" Hit return key for next span ");
             cw=getchar();  /* dummy read to accept just return */
         } else {
             spe_sleep(st->sleep_msec);
         }
         goto loop;
      }

     free(adat);
     free(tdat);
}


void spe_sleep(int msec)  // suspend msec 
{
    clock_t cst1,cst2;
    double telapsed,tsec;
    cst1 = clock();
    tsec=(double)msec/1000.0;
    if(msec <= 0) goto exit2return;
    while(1){
        cst2=clock();
        telapsed = (double)(cst2 - cst1) / CLOCKS_PER_SEC;
        if(telapsed >= tsec) break;
    }
exit2return:;

}

/* added on 2015-11-03 */


void plotspe_new(speana_param_t *st)
{
    // 2015-11-03  minor update

    int kplot, m, k;
    float ymaxdb,ymindb;

    char labfsampl[10];
#ifdef USE_GNUPLOT   // using GNUPLOT
    char ftemp4ch[4][60];
#endif

    st->xdata = (float *)malloc(st->nfft*sizeof(float));

    ymaxdb=(float)st->ymaxdb; ymindb=(float)st->ymindb;

    /* sampling frequency in text for print */
    if(st->sfreq < 1000){
        sprintf(labfsampl,"%dkHz",st->sfreq);
    } else {
        sprintf(labfsampl,"%dMHz",st->sfreq/1000);
    }

    for(kplot=0; kplot<2; kplot++){
       if(st->pmode==1 && kplot==1) goto bypass;
       if(st->pmode==2 && kplot==0) goto bypass;
       if(kplot==0) {
           strcpy(st->g_device,st->g_ps_device);
       } else {
           strcpy(st->g_device,st->g_display);
       }

#ifdef USE_GNUPLOT   // using gnuplot
       st->gp= popen(GNUPLOT_EXE,"w");
       if(st->gp==NULL) goto exitend;
       if(kplot==0) {
          //fprintf(gp,"set terminal post eps color\n");
          fprintf(st->gp,"set terminal post landscape 12 color\n");
          fprintf(st->gp,"set output \"%s\"\n",st->ps_fname);
       }
       if(kplot==1) {
          fprintf(st->gp,"set terminal %s\n",st->g_display);
       }
       fflush(st->gp);
       fprintf(st->gp,"set multiplot\n");
       fprintf(st->gp,"set size 0.5,0.5\n");

#else                // using pgplot
       //if(kplot==0) {
       //    strcpy(pgdevice,"%s/PS",st->ps_fname);
       //} else {
       //    strcpy(pgdevice,st->g_device);
       //}
       if(st->flag_g_device_open==0){
           if(cpgopen(st->g_device) < 1) goto exitend;
           st->flag_g_device_open=1;
       }
       if(strncmp(st->g_device,"/GW",3)==0) cpgeras();   // erase just in case 2007-10-26
       if(strncmp(st->g_device,"/XWIN",5)==0 ||
           strncmp(st->g_device,"/XSER",5)==0 ) { /* set background white 
                                                   and text black */
           cpgscr(0,1.0,1.0,1.0);  /* Background color */
           cpgscr(1,0.0,0.0,0.0);  /* Foreground color */
       }
       if(st->numch != 1){
           cpgsubp(2,2);  // divide plot pannel into 2X2
       }
#endif
       for(m=0; m<st->numch; m++){

#ifdef USE_GNUPLOT   // using gnuplot
           switch(m){
              case 0: fprintf(st->gp,"set origin 0.0,0.5\n"); break;
              case 1: fprintf(st->gp,"set origin 0.5,0.5\n"); break;
              case 2: fprintf(st->gp,"set origin 0.0,0.0\n"); break;
              case 3: fprintf(st->gp,"set origin 0.5,0.0\n"); break;
           }
           //fprintf(gp,"clear\n");   

#endif

           if(st->sekibun < 1.0){
               if(st->sekibun < 1.0e-3){
                  sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(usec)=%6.2f",
                     //m+1,st->adbit,labfsampl,st->sekibun*1.0e6);
                     st->chtable[m],st->adbit,labfsampl,st->sekibun*1.0e6);
               } else {
                  sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(msec)=%.3f",
                     //m+1,st->adbit,labfsampl,st->sekibun*1.0e3);
                     st->chtable[m],st->adbit,labfsampl,st->sekibun*1.0e3);
               }
           } else {
                sprintf(st->lab1,"CH#:%1d %1dbit %s sampling, Tavg(sec)=%.1f",
                   //m+1,st->adbit,labfsampl,st->sekibun);
                   st->chtable[m],st->adbit,labfsampl,st->sekibun);
           }
           for(k=0; k<st->nfft; k++) {
               st->xdata[k]=st->spows[k*st->numch+m];
           }

           if(st->amode > 0) {
                st->mode=1;
                plotautocor(st);
           } else {
                plotspe_a_new(st,ymaxdb,ymindb,m);
           }
#ifdef USE_GNUPLOT   // using gnuplot
           strcpy(ftemp4ch[m],st->ftemp);
#endif

       }

#ifdef USE_GNUPLOT   // using gnuplot
       fprintf(st->gp,"unset multiplot\n");
       fflush(st->gp);
  #ifdef WIN32
       if(kplot==1) getchar();  // this is necessary to keep display otherwise deleted
  #endif
       pclose(st->gp);
       for(m=0; m<st->numch; m++) remove(ftemp4ch[m]);

#else                // using pgplot
       /* panel undivided to date write */
       cpgsubp(1,1);
       /* date etc write */
       cpgiden();
       /* data file name write */
       cpgsch(0.7f);                 // "f" means single precision floating number
       cpgsvp(0.0,1.0,0.0,1.0);   // set viewport as all area
       cpgswin(0.0,1.0,0.0,1.0);   // combine with world corrdinates
       cpgtext(0.01f,0.01f,st->fname);

       /* close the graphics device */
       if(strncmp(st->g_device,"/GW",3)!=0){
           cpgclos();   //2007-10-26
           st->flag_g_device_open=0;
       }
#endif
       if(kplot==0){
           printf("Created PostScript file ==> %s\n",st->ps_fname);
       }
 bypass:;
    }
exitend:;
    free(st->xdata);
}

void plotspe_a_new(speana_param_t *st, float ymaxdb, float ymindb, int mch)
/*
               mode2 -- YX: axis scale mode
                          X: frequency axis mode
                             =0: log scale  =1: linear scale
                          Y: amplitude axis mode
                             =0: log scale 
                             =1: linear scale (common for 4 channels)
                                  (use ymaxdb, ymindb)
                             =2: linear scale (each max and min)
*/
{
      float fmax, fmin, fsampl, df, ymax, ymin, pmax, pmin;
      float fscale;
      float *fdat, *powdat;
      char htitl[60], xtit[60], ytit[60];
      char lab2[128];
      int n, npmax;
      int mode,ymode;

      ymode=st->mode2/10;
      mode=st->mode2 % 10;
      strcpy(htitl,"POWER SPECTRUM");
/*
      sprintf(lab2,"%s offset(sec)=%g",st->lab2,st->toffset);
      if(st->sekibun < 1.0){
          sprintf(lab2,"%s offset(msec)=%g",st->lab2,st->toffset*1.0e3);
          if(st->sekibun < 1.0e-3){
             sprintf(lab2,"%s offset(usec)=%g",st->lab2,st->toffset*1.0e6);
          }
      }
*/
      sprintf(lab2,"%s offset(msec)=%10.3f",st->lab2,st->toffset*1.0e3);

      /* frequency axis generation */

      //npmax=npos/2;
      npmax=st->nfft/2;

      fdat   = (float *)malloc(npmax*sizeof(float));
      powdat = (float *)malloc(npmax*sizeof(float));


      /* frequency axis */
      fsampl=(float)1000.0*(float)st->sfreq;
      df=fsampl/(float)st->nfft;
      if(st->sfreq > 8000){  // if sampling frequency exceeds 8 MHz
          fscale = 1.0e6;  // scaling in MHz
          strcpy(xtit,"FREQUENCY (MHz)");
      } else {
          fscale = 1000.0; // scaling in kHz
          strcpy(xtit,"FREQUENCY (kHz)");
      }
      if(mode == 0) {    // log scale
#ifdef USE_GNUPLOT   // using gnuplot
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=(float)(df/fscale);
         for(n=0; n<npmax; n++)fdat[n]=(float)((float)n*df/fscale); 
#else // use PGPLOT
         fmax=(float)log10(fsampl/(2.0*fscale));
         fmin=(float)log10(df/fscale);
         for(n=0; n<npmax; n++) fdat[n]=(float)log10((float)n*df/fscale);
#endif
      } else {          //  linear scale
         fmax=(float)(fsampl/(2.0*fscale));
         fmin=0.0;
         if(st->f1khz != 0.0) fmin=(float)(st->f1khz/(fscale/1000.0));
         if(st->f2khz != 0.0) fmax=(float)(st->f2khz/(fscale/1000.0));
         for(n=0; n<npmax; n++){fdat[n]=(float)((float)n*df/fscale); }
      }

      /* amplitude axis */

      /* data copy, if necessary convert to linear scale */
      if(ymode == 0){    // log scale
         for(n=0; n<npmax; n++) powdat[n]=st->xdata[n];
      } else {   // linear scale
         /* convert to linear scale */
         for(n=0; n<npmax; n++) powdat[n]=(float)pow(10.0,st->xdata[n]/10.0);
      }

      /* data max, min check */
      for(n=0; n<npmax; n++){
         //powdat[n]=spow[n];
         if(n==0){
            pmax=powdat[n];
            pmin=powdat[n];
         } else {
            if(powdat[n] > pmax) pmax=powdat[n];
            if(powdat[n] < pmin) pmin=powdat[n];
         }
      }

      if(ymode == 0){    // log scale
         strcpy(ytit,"POWER (dBm)");
         if( st->adbit==1 ) strcpy(ytit,"POWER (dB)"); 
         ymax=10.0;
         ymin=-80.0;
         //if (st->adbit >= 8 ) ymin=-100;     // extend dynamic range (2015-10-27 to support AD 16 bits)
         if(st->maxdbm!=0.0){     // 2014-10-29
             ymax=(float)st->maxdbm;
         }
         if(st->mindbm!=0.0){     // 2014-10-29
             ymin=(float)st->mindbm;
         }

      } else {   // linear scale
         strcpy(ytit,"POWER (linear)");
         ymax=pmax;
         ymin=pmin;
         if(ymode==1){
            ymax=(float)pow(10.0,ymaxdb/10.0);
            ymin=(float)pow(10.0,ymindb/10.0);
         }
      }

#ifdef USE_GNUPLOT   // using gnuplot

      /* define coordinate range of graph */
      if(mode == 0){   
           fprintf(st->gp,"set logscale x\n");
      } else {         
           fprintf(st->gp,"unset logscale\n");
      }

      fprintf(st->gp,"set xrange [%g:%g]\n",fmin,fmax);
      fprintf(st->gp,"set yrange [%g:%g]\n",ymin,ymax);
      
      /* label the axes */
      fprintf(st->gp,"set xlabel \"%s\"\n",xtit);
      fprintf(st->gp,"set ylabel \"%s\"\n",ytit);
      //fprintf(st->gp,"set title \"%s\"\n",htitl);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
      //fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      if(strncmp(st->g_device,"windows",7)==0 || 
          strncmp(st->g_device,"x11",3)==0 ){ // display
         fprintf(st->gp,"set title \"%s\"\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.05 center\n",st->lab2);
      } else { // postscript
         fprintf(st->gp,"set title \" \"\n");  // this is to keep dummy space
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.14 center\n",htitl);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.08 center font \"Helvetica,10\"\n",st->lab1);
         fprintf(st->gp,"set label \"%s\" at graph 0.5, 1.03 center font \"Helvetica,10\"\n",st->lab2);
      }

      /* check data */
      if (pmax == pmin){
          fprintf(st->gp,"set label \"NO TIME VARIABLE\" at graph 0.5, 0.52 center\n");
          fprintf(st->gp,"set label \"COMPONENT FOUND\" at graph 0.5, 0.48 center\n");
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  
          gnline2(st->gp,2,fdat,powdat,st->ftemp); 
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          gnline2(st->gp,npmax,fdat,powdat,st->ftemp); 

      }
      fprintf(st->gp,"unset label\n");  // this is necessary otherwise 
                                    // CH2,3,4 label are overlapped 

#else  // using PGPLOT

      cpgsch(2.0);  // set text height 2.0  (default is 1)
      cpgslw(2);    // set line width

      cpgpanl((mch % 2)+1,(mch/2+1));  // when use this, should not use cpgenv
                                           //    mch    ix      iy        cpgpanl(ix,iy)
                                           //     0      1       1
                                           //     1      2       1
                                           //     2      1       2
                                           //     3      2       2

      cpgeras();  // doesn't work properly with cpgenv

      cpgvstd();
      /* define coordinate range of graph */
      if(mode == 0){   
        // cpgenv(fmin, fmax, ymin, ymax, 0, 10);   // log scale
            cpgswin(fmin, fmax, ymin, ymax);          //  these two lines work as same as cpgenv()
           cpgbox("BCNTS2",0.0,0,"BCNTS",0.0,0);        //
      } else {         
         //cpgenv(fmin, fmax, ymin, ymax, 0, 0);   // linear scale
           cpgswin(fmin, fmax, ymin, ymax);          //  these two lines work as same as cpgenv()
           cpgbox("BCNTS",0.0,0,"BCNTS",0.0,0);        //
       }
      
      /* label the axes */
      cpglab(xtit,ytit,htitl);

      /* sub label write*/
      cpgsch(1.5);
      cpgmtxt("T",1.5,0.5,0.5,st->lab1);
      //cpgmtxt("T",0.5,0.5,0.5,st->lab2);
      cpgmtxt("T",0.5,0.5,0.5,lab2);
      cpgsch(2.0);

      /* check data */
      if (pmax == pmin){
          cpgmtxt("B",-6.0,0.5,0.5,"NO TIME VARIABLE");
          cpgmtxt("B",-4.0,0.5,0.5,"COMPONENT FOUND");
          /* label the axes */
          
      } else {

          /* plot the line graph */
          fdat[0]=fdat[1];
          powdat[0]=powdat[1];  // to avoid ugry plot
          //cpgslw(1);    // set line width
          cpgline(npmax,fdat,powdat); 

      }
   
#endif

     free(powdat);
     free(fdat);
}
